/*
 * coordinate_transform.cpp
 *
 *  Created on: Aug 11, 2017
 *      Author: julien
 */

#include "coordinate_transform.hpp"

using namespace cv;
using namespace std;

CoordinateTransform::CoordinateTransform(float camera_height,
										 float camera_angle,
										 float aperture_angle_x,
										 float aperture_angle_y,
										 Size2i picture_size,
										 Function::pPoint camera_translation) :
camera_height(camera_height),
height_squared(camera_height*camera_height),
camera_angle(camera_angle),
aperture_angle_x(aperture_angle_x),
aperture_angle_y(aperture_angle_y),
picture_height(picture_size.height),
picture_width(picture_size.width),
camera_translation(camera_translation),
f(20)
{}


Point CoordinateTransform::pixel2bottom_coord(Point pixel_point){
	return Point(pixel_point.x - picture_width/2, picture_height - pixel_point.y);
}

Point CoordinateTransform::bottom_coord2pixel(Point centric_point){
	return Point(centric_point.x + picture_width/2, picture_height - centric_point.y);
}


Function::pPoint CoordinateTransform::pixel2ego_coord(Point input){

	Point coordinate = pixel2bottom_coord(input);
	double sy = camera_height / f.sinus(camera_angle - aperture_angle_y*((coordinate.y/picture_height) - 0.5));

	double y = sqrt(sy*sy - height_squared);
	double x = sy * f.tangens(aperture_angle_x * coordinate.x/picture_width);

	return Function::pPoint(y + camera_translation.y, -(x + camera_translation.x));

}

Function::pPoint CoordinateTransform::ego_coord2pixel(Function::pPoint input){

	double x = -(input.y + camera_translation.x);
	double y = input.x - camera_translation.y;

	double sy = sqrt(y * y + height_squared);
	double py = picture_height *
				( ( (camera_angle - f.arc_sinus(camera_height/sy) ) / aperture_angle_y) + 0.5);
	double px = (picture_width / aperture_angle_x) * f.arc_tangens(x / sy);

	Point p(px, py);
	Point ret = bottom_coord2pixel(p);

	return Function::pPoint(ret.x, ret.y);
}

void CoordinateTransform::transf(vector<Function::pPoint>* input_lane,
								 vector<Function::pPoint>* ouput_lane,
								 bool input_is_pixel){

	int size = input_lane->size();

	*ouput_lane = vector<Function::pPoint>(size);

	Function::pPoint* point;

	for(int i = 0; i < size; i++){

		point = &input_lane->at(i);

		if(abs(point->x) > 4000) // TODO in abhängigkeit der Bildgröße und des boolschen wertes
			continue;
		else if(input_is_pixel)
			ouput_lane->at(i) = pixel2ego_coord(Point(point->x, point->y));
		else
			ouput_lane->at(i) = ego_coord2pixel(*point);
	}

}


void CoordinateTransform::transform(vector<Function::pPoint>* input_l,
								    vector<Function::pPoint>* ouput_l,
									vector<Function::pPoint>* input_r,
									vector<Function::pPoint>* ouput_r,
									bool input_is_pixel){

	transf(input_l, ouput_l, input_is_pixel);
	transf(input_r, ouput_r, input_is_pixel);
}



